@@ -704,7 +704,13 @@ AccessibleEvent_unref (const AccessibleEvent *e)
 	{
 	  event->ref_count--;
 	  if (event->ref_count < 1)
-	    cspi_internal_event_remove (event);
+            {
+	      cspi_internal_event_remove (event);
+              g_free (e->type);
+              Accessible_unref (e->source);
+              CORBA_free (event->data);
+              g_free (e);
+            }
 	}
     }
 }
